// Proyecto: NetAnalyzer
// Datos: Codigo de Pruebas para Puertos de Entra y Salida: Bateria, Botones, Buzzer, Led, Rotary Encoder
// Autor: Luciano Lagassa <info@lucianolagassa.com.ar>
// Web: https://wwww.lucianolagassa.com.ar
// Basado en "LiquidCrystal Library - Hello World" de Arduino Team (https://www.arduino.cc/en/Tutorial/HelloWorld) 
// Basado en "How Rotary Encoder Works" de Dejan Nedelkovski (http://howtomechatronics.com/tutorials/arduino/rotary-encoder-works-use-arduino/)
// Basado en testDHCP de EtherCard Library (https://github.com/jcw/ethercard/)
//
// LiquidCrystal Library
#include <LiquidCrystal.h>
const int rs = 8, en = 7, d4 = 6, d5 = 5, d6 = 4, d7 = 3;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);
// EtherCard Library
#include <EtherCard.h>
static byte mymac[] = { 0x74,0x69,0x69,0x2D,0x30,0x31 };
byte Ethernet::buffer[700];
//
// Variables
String Version = "v2018-01-04-0825";
boolean DevMode = false;              // Activar Modo Desarrollo
int LedPin = 16;                      // Led Indicador de Estado en Puerto A2
int BuzzerPin = 14;                   // Buzzer Indicador de Estado en Puerto A0
int BatteryPin = A6;                  // Puerto de Lectura de la Bateria en Puerto A6
int BatteryMin = 650;                 // Valor Minimo de Bateria 9V
int OKButtonPin = A7;                 // Boton OK en Puerto A7
int BackButtonPin = 2;                // Boton Atras en Puerto 2
int RotaryAPin = 15;                  // Rotary Encoder A en Puerto A1
int RotaryBPin = 17;                  // Rotary Encoder B en Puerto A3
volatile int RotaryCount = 0;         // Rotary Encoder Counter
int RotaryLast;                       // Rotary Encoder Valor Previo
int RotaryRead;                       // Rotary Encoder Valor Actual
char* Temporal = "";                  // Variable para Converciones
char Salida[64] = {};                 // Variable para Converciones
boolean NetState = false;             // Estado de Red
volatile boolean MainMenu = false;    // Menu de Sistema
boolean NetReload = false;            // Recargar Red
boolean SystemStart = false;          // Variable de Sistema
int MenuMin = 0;                      // Valor Minimo de Menu
int MenuMax = 5;                      // Valor Maximo de Menu
int MenuItem = 0;                     // Item Seleccionado de Menu
int LCDCounter = 0;                   // Contador de Scroll del LCD
long T0 = 0;                          // Variable de Control en Interupciones

void setup()
{
  Serial.begin(9600);
  Serial.println("NetAnalyzer "+Version);
  Serial.println("Luciano Lagassa <info@lucianolagassa.com.ar>");
  // AppName and Version
  LCDText("NetAnalyzer",Version,0,false);
  delay(500);
  // PinMode Set
  pinMode(LedPin, OUTPUT);
  pinMode(OKButtonPin, INPUT);
  pinMode(BackButtonPin, INPUT);
  pinMode(RotaryAPin, INPUT);
  pinMode(RotaryBPin, INPUT);
  // Check Battery Level
  if (analogRead(BatteryPin) > BatteryMin)
  {
    // Battery Level OK
    // Buzzer Test
    Serial.println("Buzzer Test");
    tone(BuzzerPin,900,200);
    delay(300);
    tone(BuzzerPin,900,200);
    // LedPin Test
    Serial.println("LedPin Test");
    digitalWrite(LedPin, HIGH);
    delay(300);
    digitalWrite(LedPin, LOW);
    delay(300);
    digitalWrite(LedPin, HIGH);
    delay(300);
    digitalWrite(LedPin, LOW);
    // Read Rotary Encoder Inicial State
    RotaryLast = digitalRead(RotaryAPin);
    if (DevMode == true)
    {
      // Start in Developer Mode
      LCDText("NetAnalyzer","Developer Mode",900,true);
      delay(500);
    }
    // System Start
    LCDText("NetAnalyzer","System Starting",0,false);
    SystemStart = true;
    delay(500);
    NetInit();
  }
  else
  {
    if (DevMode == true)
    {
      // Start in Developer Mode
      LCDText("NetAnalyzer","Developer Mode",900,true);
      delay(800);
      LCDText("NetAnalyzer","System Starting",0,false);
      SystemStart = true;
      delay(500);
      NetInit();
    }
    else
    {
      // Battery level Low, the system will not start
      Serial.println("Battery level Low, The System will not Start");
      LCDText("NetAnalyzer","Battery LOW",200,false);
      SystemStart = false;
    }
  }
  attachInterrupt(0, BackInt, RISING);
}

void loop()
{
  if (SystemStart == true)
  {
    if (MainMenu == false)
    {
      MenuMin = 0;
      MenuMax = 5;
    }
    else
    {
      MenuMin = 0;
      MenuMax = 7;
    }
    RotaryRead = digitalRead(RotaryAPin);
    if (RotaryRead != RotaryLast)
    {
      if (digitalRead(RotaryBPin) != RotaryRead)
      {
        if (RotaryCount > MenuMin)
        {
          RotaryCount --;
        }
      }
      else
      {
        if (RotaryCount < MenuMax)
        {
          RotaryCount ++;
        }
      }
      if (MainMenu == true)
      {
        switch (RotaryCount)
        {
          case 1: // Ping to Gateway
            LCDText("System Menu","Ping to Gateway",0,false);
            MenuItem = RotaryCount;
            break;
          case 2: // Ping to DNS
            LCDText("System Menu","Ping to DNS",0,false);
            MenuItem = RotaryCount;
            break;
          case 3: // Ping to Google DNS
            LCDText("System Menu","Ping to Google",0,false);
            MenuItem = RotaryCount;
            break;
          case 4: // Rescan Network
            LCDText("System Menu","Rescan Network",0,false);
            MenuItem = RotaryCount;
            break;
          case 5: // Send Network Info to Email
            LCDText("System Menu","Send Info",0,false);
            MenuItem = RotaryCount;
            break;
          case 6: // About
            LCDText("System Menu","About",0,false);
            MenuItem = RotaryCount;
            break;
          case 7: // Last Item
            tone(BuzzerPin,1000,200);
            digitalWrite(LedPin, HIGH);
            delay(300);
            digitalWrite(LedPin, LOW);
            RotaryCount = 6;
            break;
        }
      }
      else
      {
        switch (RotaryCount)
        {
          case 1: // Show IP Address
            Temporal = "%d.%d.%d.%d";
            sprintf(Salida, Temporal, ether.myip[0],  ether.myip[1],  ether.myip[2],  ether.myip[3]);
            LCDText("IP Address",Salida,0,false);
            break;
          case 2: // Show Network Mask
            Temporal = "%d.%d.%d.%d";
            sprintf(Salida, Temporal, ether.netmask[0],  ether.netmask[1],  ether.netmask[2],  ether.netmask[3]);
            LCDText("Network Mask",Salida,0,false);
            break;
          case 3: // Show Gateway Address
            Temporal = "%d.%d.%d.%d";
            sprintf(Salida, Temporal, ether.gwip[0],  ether.gwip[1],  ether.gwip[2],  ether.gwip[3]);
            LCDText("Gateway Address",Salida,0,false);
            break;
          case 4: // Show DNS Address
            Temporal = "%d.%d.%d.%d";
            sprintf(Salida, Temporal, ether.dnsip[0],  ether.dnsip[1],  ether.dnsip[2],  ether.dnsip[3]);
            LCDText("DNS Address",Salida,0,false);
            break;
        }
      }
      RotaryLast = RotaryRead;
    }
    // OKButton
    if (analogRead(OKButtonPin) > 100)
    {
      if (MainMenu == true)
      {
        switch (RotaryCount)
        {
          case 1: // Ping to Gateway
            LCDText("Ping to Gateway","In Development",900,true);
            MainMenu = false;
            RotaryCount = 0;
            break;
          case 2: // Ping to DNS
            LCDText("Ping to DNS","In Development",900,true);       
            MainMenu = false;
            RotaryCount = 0;
            break;
          case 3: // Ping to Google DNS
            LCDText("Ping to Google","In Development",900,true);
            MainMenu = false;
            RotaryCount = 0;
            break;
          case 4: // Rescan Network
            LCDText("Rescan Network","In Development",900,true);
            //LCDText("NetAnalyzer","System Starting",900,true);
            MainMenu = false;
            RotaryCount = 0;
            //NetInit();
            break;
          case 5: // Send Network Info to Email
            LCDText("Send Info","In Development",900,true);
            MainMenu = false;
            RotaryCount = 0;
            break;
          case 6: // About
            LCDText("NetAnalyzer","wwww.lucianolaga",900,true);
            delay(500);
            LCDText("NetAnalyzer","www.lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"ww.lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"w.lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer",".lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"ucianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"cianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","ianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","anolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer "+Version,"nolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","anolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","ianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"cianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"ucianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer",".lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"w.lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText(Version,"ww.lucianolagassa.com.ar",0,true);
            delay(500);
            LCDText("NetAnalyzer","wwww.lucianolaga",0,true);
            delay(800);
            LCDText("NetAnalyzer",Version,900,true);
            delay(800);
            MainMenu = false;
            RotaryCount = 0;
            break;
        }
      }
      else
      {
        LCDText("System Menu","Select Tool",900,true);
        MainMenu = true;
        RotaryCount = 0;
      }
    }
    // BackButton
    if (digitalRead(BackButtonPin) == HIGH)
    {
      if (MainMenu == true)
      {
        MainMenu = false;
        tone(BuzzerPin,900,300);
        digitalWrite(LedPin, HIGH);
        delay(500);
        digitalWrite(LedPin, LOW);
      }
      RotaryCount = 0;
      //LCDText("Rescan Network","In Development",900,true);
      LCDText("NetAnalyzer","Move to Info",900,true);
      //LCDText("NetAnalyzer","System Starting",0,false);
      //NetInit();
    }
  }
}

void NetInit()
{
  digitalWrite(LedPin, HIGH);
  if (ether.begin(sizeof Ethernet::buffer, mymac) == 0)
  {
    LCDText("NetAnalyzer","Ethernet [Error]",200,true);
  }
  else
  {
    if (NetReload == true)
    {
      const static uint8_t ip[] = {192,168,0,100};
      const static uint8_t gw[] = {192,168,0,254};
      const static uint8_t dns[] = {192,168,0,1};
      if(!ether.staticSetup(ip, gw, dns));
      {
        Serial.println(F("Setting Static IP Address"));
      }
      NetReload = true;
    }
    if (!ether.dhcpSetup())
    {
      LCDText("NetAnalyzer","DHCP [Failed]",200,true);
      NetState = false;
    }
    else
    {
      LCDText("NetAnalyzer","DHCP [OK]",900,true);
      delay(500);
      LCDText("NetAnalyzer","Move to Info",900,true);
      NetState = true;
    }
  }
  delay(1000);
  digitalWrite(LedPin, LOW);
}

void LCDText(String Text1, String Text2, int Sound, boolean Led)
{
  if (Sound > 0)
  {
   tone(BuzzerPin,Sound,300);
  }
  if (Led == true)
  {
    digitalWrite(LedPin, HIGH);
    delay(500);
    digitalWrite(LedPin, LOW);
  }
  Serial.println(Text1+" - "+Text2);
  lcd.clear();
  lcd.begin(16, 2);
  lcd.print(Text1);
  lcd.setCursor(0, 1);
  lcd.print(Text2);
  delay(100);
}

void BackInt()
{
  if (MainMenu == true)
  {
    digitalWrite(LedPin, HIGH);
    if ( millis() > T0  + 500)
    {
      digitalWrite(LedPin, LOW);
      MainMenu = false;
    }
  }
  lcd.clear();
  lcd.begin(16, 2);
  lcd.print("NetAnalyzer");
  lcd.setCursor(0, 1);
  lcd.print("Move to Info");
  RotaryCount = 0;
  if ( millis() > T0  + 500)
  {
    tone(BuzzerPin,900,300);
  }
}

